<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://www.epubsystems.com
 * @since      1.0.0
 *
 * @package    Epubsystems_woo
 * @subpackage Epubsystems_woo/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Epubsystems_woo
 * @subpackage Epubsystems_woo/admin
 * @author     Nir Louk <looknear@gmail.com>
 */
class Epubsystems_woo_Admin {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of this plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct($plugin_name, $version) {

        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_styles() {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Epubsystems_woo_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Epubsystems_woo_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */
        wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/epubsystems_woo-admin.css', array(), $this->version, 'all');
    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts() {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Epubsystems_woo_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Epubsystems_woo_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */
        wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/epubsystems_woo-admin.js', array('jquery'), $this->version, false);
    }

    public function add_plugin_admin_menu() {

        /*
         * Add a settings page for this plugin to the Settings menu.
         *
         * NOTE:  Alternative menu locations are available via WordPress administration menu functions.
         *
         *        Administration Menus: http://codex.wordpress.org/Administration_Menus
         *
         */

        add_management_page('EpubSystems WooSell Settings', 'eBook WooSell', 'manage_options', $this->plugin_name, array($this, 'display_plugin_setup_page'));
    }

    public function add_action_links($links) {
        /*
         *  Documentation : https://codex.wordpress.org/Plugin_API/Filter_Reference/plugin_action_links_(plugin_file_name)
         */
        $settings_link = array(
            '<a href="' . admin_url('options-general.php?page=' . $this->plugin_name) . '">' . __('Settings', $this->plugin_name) . '</a>',
        );
        return array_merge($settings_link, $links);
    }

    /**
     * Render the settings page for this plugin.
     *
     * @since    1.0.0
     */
    public function display_plugin_setup_page() {
        include_once( 'partials/epubsystems_woo-admin-display.php' );
    }

    public function options_update() {
        register_setting($this->plugin_name, $this->plugin_name, array($this, 'validate'));
    }

    public function validate($input) {
        // All checkboxes inputs
        $valid = array();

        //check values
        $valid['es_publisher_email'] = (isset($input['es_publisher_email']) && !empty($input['es_publisher_email'])) ? $input['es_publisher_email'] : "";
        $valid['es_title'] = (isset($input['es_title']) && !empty($input['es_title'])) ? $input['es_title'] : "";
        $valid['es_message'] = (isset($input['es_message']) && !empty($input['es_message'])) ? $input['es_message'] : "";
        $valid['es_link_text'] = (isset($input['es_link_text']) && !empty($input['es_link_text'])) ? $input['es_link_text'] : "";
        $valid['es_link_href'] = (isset($input['es_link_href']) && !empty($input['es_link_href'])) ? $input['es_link_href'] : "";
        $valid['es_gift_text'] = (isset($input['es_gift_text']) && !empty($input['es_gift_text'])) ? $input['es_gift_text'] : "";
        $valid['es_email_placeholder'] = (isset($input['es_email_placeholder']) && !empty($input['es_email_placeholder'])) ? $input['es_email_placeholder'] : "";
        $valid['es_show_in_orders_list'] = isset($input['es_show_in_orders_list']) && $input['es_show_in_orders_list']==='1' ? 1 : 0;

        return $valid;
    }

    //show custom data on order page in admin view
    public function admin_show_order_meta($order) {

        echo '<p><b>' . _e('EpubSystems Cloud delivery', $this->plugin_name) . '</b></p>';

        $this->show_webhook_status($order, true);

    }


    ///////// for ADMIN MANUALLY creating or updating existing order  //////////////
    public function admin_update_order_meta ($post_id, $post){
        error_log('# ES : created or updated order !');

        // Get an instance of the WC_Order object (in a plugin)
        $order = new WC_Order( $post_id );
        //update the es_publisher_email
        $options = get_option($this->plugin_name);
        if (!empty($options) && !empty($options['es_publisher_email'])) {
            //admin set his email in the admin dashboard
            update_post_meta($order->get_id(), 'es_publisher_email', $options['es_publisher_email']);
        } else {
            //just send the wordpress admin email
            update_post_meta($order->get_id(), 'es_publisher_email', get_option('admin_email'));
        }

        // !!!!!! es_user_email need to be set manually by the ADMIN !!!
        if (!$order->meta_exists('es_user_email')){
            update_post_meta($order->get_id(), 'es_user_email', $order->get_billing_email());
            //error_log("adding user from billing email: ". $order->get_billing_email());
        }

        //add info to each item
        $items = $order->get_items();
        foreach ( $items as $item ) {
            $product = $item->get_product();

            // DBG the raw product
            //error_log( print_r($product, true) );
  
            //error_log("delete keys ===============");
            //$item->delete_meta_data('is_virtual');
            //$item->save_meta_data();
            
            //it's always in the product if it exist. no virtual to variations
            if (!$item->meta_exists('is_virtual')){
                //error_log("add is_virtual ===============");
                $item->add_meta_data('is_virtual', $product->is_virtual(), true);
            }else{
                //error_log("update is_virtual ===============");
                $item->update_meta_data('is_virtual', $product->is_virtual(), true);
            }

            //============ is_es_ebook =========================
            //in case simple product, we saved it in product
            $is_es_ebook = 'no';
            if ($product->meta_exists('_is_es_ebook')){
                $is_es_ebook = $product->get_meta('_is_es_ebook');
                //error_log(" _is_es_ebook | in simple product  : ".$is_es_ebook);  
            }else{
                 //maybe variation, get from post data
                $is_es_ebook = get_post_meta($item->get_id(), '_is_es_ebook', true);
                //error_log(" _is_es_ebook | in variation : ".$is_es_ebook);
                
            }
            //Set the value in the order item
            if (!$item->meta_exists('is_es_ebook')){
                //error_log("add is_es_ebook : ". $is_es_ebook);
                $item->add_meta_data('is_es_ebook', $is_es_ebook, true);
            }else{
                //error_log("update is_es_ebook : ". $is_es_ebook);
                $item->update_meta_data('is_es_ebook', $is_es_ebook, true);
            }

            //============ es_days_valid =========================
            //in case simple product, we saved it in product
            $es_days_valid = 0;
            if ($product->meta_exists('_es_days_valid')){
                $es_days_valid = intval($product->get_meta('_es_days_valid'));
                //error_log(" _es_days_valid | in simple product  : ".$es_days_valid);  
            }else{
                 //maybe variation, get from post data
                $es_days_valid = intval(get_post_meta($item->get_id(), '_es_days_valid', true));
                //error_log(" _es_days_valid | in variation : ". $es_days_valid);
                
            }
            //Set the value in the order item
            if (!$item->meta_exists('es_days_valid')){
                //error_log("add es_days_valid : ". $es_days_valid);
                $item->add_meta_data('es_days_valid', $es_days_valid, true);
            }else{
                //error_log("update es_days_valid : ". $es_days_valid);
                $item->update_meta_data('es_days_valid', $es_days_valid, true);
            }

            //============ es_cloud_book_code =========================
            //in case simple product, we saved it in product
            $es_cloud_book_code = 'no';
            if ($product->meta_exists('_es_cloud_book_code')){
                $es_cloud_book_code = $product->get_meta('_es_cloud_book_code');
                //error_log(" _es_cloud_book_code | in simple product  : ".$es_cloud_book_code);  
            }else{
                 //maybe variation, get from post data
                $es_cloud_book_code = get_post_meta($item->get_id(), '_es_cloud_book_code', true);
                //error_log(" _es_cloud_book_code | in variation : ".$es_cloud_book_code);
                
            }
            //Set the value in the order item
            if (!$item->meta_exists('es_cloud_book_code')){
                //error_log("add es_cloud_book_code : ". $es_cloud_book_code);
                $item->add_meta_data('es_cloud_book_code', $es_cloud_book_code, true);
            }else{
                //error_log("update es_cloud_book_code : ". $es_cloud_book_code);
                $item->update_meta_data('es_cloud_book_code', $es_cloud_book_code, true);
            }
            
            /////////////MUST CALL IT !!!! ////////////
            $item->save_meta_data();

        }

         //error_log("epub_woo : admin_update_order_meta");
    }


    /////////////// orders column title /////////////////
    public function admin_add_orders_columns_title ($columns){
      $options = get_option($this->plugin_name);
      $show_in_list = !isset($options['es_show_in_orders_list']) || $options['es_show_in_orders_list'];
      if (!$show_in_list){
        return $columns;
      }

      $new_columns = array();
      foreach ( $columns as $key => $name ) {
          $new_columns[ $key ] = $name;
          if ( 'order_status' === $key ) {
              $new_columns['order-es-webhook-status'] = __('ES Cloud', $this->plugin_name);
          }
      }
      return $new_columns;
    }

    /////////////// orders column data /////////////////
    public function admin_set_orders_columns_data( $colkey ) {
      $options = get_option($this->plugin_name);
      $show_in_list = !isset($options['es_show_in_orders_list']) || $options['es_show_in_orders_list'];
      if (!$show_in_list){
        return;
      }

      global $the_order; // the global order object
      if ($colkey === 'order-es-webhook-status'){
          $this->show_webhook_status($the_order, false);
      }
    }

    //used for orders list and to order page with extra data
    private function show_webhook_status($the_order, $with_data){
        
        
        $webhook_title = "Order await send to ES";
        $webhook_time = get_post_meta($the_order->get_id(), 'es_webhook_time', true);

        $webhook_status = get_post_meta($the_order->get_id(), 'es_webhook_status', true);
        if (isset($webhook_status)){
            //if we updated it - it was sent in some point
            $webhook_title = get_post_meta($the_order->get_id(), 'es_webhook_title', true);
        }else{
            //default: webhook didnt sent yet.
            $webhook_status = ES_WEBHOOK_STATUS_AWAIT; //'AWAIT'
        }
        $webhook_status_css_class = "es_webhook_status_".$webhook_status;
        
        //indicate if is a gift
        $is_gift = get_post_meta($the_order->get_id(), 'es_is_gift', true);
        if (!empty($is_gift) && $is_gift){
            echo '<span title="As Gift" class="dashicons dashicons-buddicons-community"></span>';
        }

        $client_email_label = get_post_meta($the_order->get_id(), 'es_user_email', true);
        if (empty($client_email_label)){
            $client_email_label = $webhook_title;
        }
        echo '<span title="'.$webhook_title.'" style="padding: 4px;" class="'.$webhook_status_css_class.'">' . $client_email_label .'</span>';

        if ($with_data){
            //time
            echo '</br><span>' . $webhook_time .'</span>';

            $webhook_data = get_post_meta($the_order->get_id(), 'es_webhook_data', true);
            if (!empty($webhook_data)){
                echo '</br><span title="more info" class="es_webhook_data">' . $webhook_data .'</span>';
            }
        }
    }



    /////////////// handle webhook response  //////////////////
    public function es_get_webhook_response ($http_args, $response, $duration, $arg, $webhook_id){
        // error_log("=== http_args:".print_r($http_args, true));
         $webhook_subject = $http_args['headers']['X-WC-Webhook-Topic'];
         if ($webhook_subject === 'order.updated'){
            error_log("# ES cloud ".$webhook_subject." : Analyzing response... ");
             $this->order_update_webhook($http_args, $response, $duration, $arg, $webhook_id);
         }else{
            error_log("# ES cloud Msg Discarded! responde only to order.updated");
         }

     }

     //save it in order meta. to monitor latest response
     private function order_update_webhook ($http_args, $response, $duration, $arg, $webhook_id){

        //error_log("=== arg:".$arg); //order id on order update. product id in product update.
        //error_log("=== duration:".$duration);//time took
        //error_log("=== webhook_id:".$webhook_id);//order in list. useless.
        //error_log("=== response:".print_r($response['body'], true));

        //$httpBodyJson = json_decode( $http_args['body'] );
        //error_log("status: ".$httpBodyJson->status);
        //if ($httpBodyJson->status === 'pending'){
        //    update_post_meta($arg, 'es_webhook_status', ES_WEBHOOK_STATUS_WARN);
        //    return;
        //}

        $time = current_time("Y-m-d H:i:s");//wordpress gmt adjusted
        //clean the data
        $responseBodyJson = null;
        update_post_meta($arg, 'es_webhook_data', "");
        if (is_array($response) && !empty($response['body'])) {
            $responseBodyJson = json_decode( $response['body'] );
            //check if maybe hebrew made a mess?
            //https://stackoverflow.com/questions/30830307/getting-hebrew-json-to-php-file
            if ( is_null( $responseBodyJson) ){
                error_log("ES convert hebrew json");
                $content=iconv("UTF-16", "UTF-8", $response['body']);
                $responseBodyJson = json_decode( $content );
            }
        }
        
        if ( !is_null( $responseBodyJson) ){
            $webhook_status = isset($responseBodyJson->errorCode)?$responseBodyJson->errorCode:CLOUD_API_FAIL;
            update_post_meta($arg, 'es_webhook_status', $webhook_status);
            update_post_meta($arg, 'es_webhook_title', $responseBodyJson->msg);
            if (is_countable($responseBodyJson->data) && count($responseBodyJson->data) > 0){
                $data_msgs = implode("</br>", $responseBodyJson->data);
                update_post_meta($arg, 'es_webhook_data', $data_msgs);
            }  
        }else{
            update_post_meta($arg, 'es_webhook_status', CLOUD_API_FAIL);
            update_post_meta($arg, 'es_webhook_title', "Server Faild on communicate to Cloud Manager");
        }
        update_post_meta($arg, 'es_webhook_time', $time);

     }

     //not in use
     //public function es_disable_async_webhook(){
     //    return false;
     //}
     
     //create the modal of the code duplicator
    public function open_thickbox_modal_convert_to_ebook(){
        //print_r($_GET);
        //$coupon_id = intval($_GET['coupon_id']);
        
        //no need for thickbox? define( 'IFRAME_REQUEST', true );
        iframe_header();
        include_once( 'partials/modal-convert-to-ebook.php' );
        iframe_footer();
        exit;
    }
    
    public function open_thickbox_modal_copy_sku_to_ebook_code(){
        iframe_header();
        include_once( 'partials/modal-copy-sku-to-ebook-code.php' );
        iframe_footer();
        exit;
    }
    
    
    //generate the coupons
    public function es_woo_sell_ajax_endpoint(){
        //error_log ("es_woo_sell_ajax_endpoint");
        global $wpdb; // this is how you get access to the database

        //not user input, but lets clean it
        $func = sanitize_text_field($_POST['fn']);
        //will verify on switch 
        
        //user input. clean spaces.
        //$product_type = sanitize_text_field(preg_replace('/\s+/', '', $_POST['product_type']));
        //if (empty($code_prefix)){
        //    echo json_encode(array('message' => "Must type codes prefix!", 'data' => array()));
        //    wp_die();
        //}
        
        
        switch ($func) {
            case "virtual2ebook":
                $this->run_es_virtual_to_ebook();
                break;
            
            case "sku2ebookcode":
                $this->run_es_sku_to_ebook_code();
                break;
            
            default:
                echo json_encode(array('message' => "Action not detected", 'data' => array()));
                wp_die();
                break;
        }
        
    }
     
     //add eBook - option to simple product
     public function admin_add_es_ebook_simple_product_option($product_type_options){
         //error_log(print_r($product_type_options, true));
         //nlook: id must be index with _
         $product_type_options['is_es_ebook'] = array(
		'id'            => '_is_es_ebook',
		'wrapper_class' => 'show_if_simple',
		'label'         => __( 'eBook', 'woocommerce' ),
		'description'   => __( 'eBook that is remotly sold via EpubSystems', 'woocommerce' ),
		'default'       => 'no'
	);

	return $product_type_options;
     }
     
     public function es_add_simple_product_custom_fields (){
        //https://www.cloudways.com/blog/add-custom-product-fields-woocommerce/
        // Number Field
        woocommerce_wp_text_input(
            array(
              'id'          => '_es_days_valid',
              'label'       => __( 'eBook Validity (days)', 'woocommerce' ),
              'description'   => __( 'Landing for X days (0-zero indicate a regular sell)', 'woocommerce' ),
              'placeholder' => 'days valid',
              'type' => 'number',
              'custom_attributes' => array(
                    'step'  => '1',
                    'min' => '0'
                  ) 
            )
         );
        
        woocommerce_wp_text_input(
            array(
              'id'          => '_es_cloud_book_code',
              'label'       => __( 'ES cloud book sku *', 'woocommerce' ),
              'placeholder' => 'unique code of eBook on EpubSystems cloud',
              'desc_tip'    => 'true'
            )
          );
        
    }
     
     //save simple product options
     public function es_save_simple_product_option($product){
         if( $product->is_type( 'simple' ) ){ 
             //is_es_ebook
            $woocommerce_checkbox = isset($_POST['_is_es_ebook']) ? 'yes' : 'no';
            //error_log("Saving simple is_es_ebook data: ".$woocommerce_checkbox);
            $product->update_meta_data( '_is_es_ebook', $woocommerce_checkbox );
            $days_valid = 0;
            $cloud_book_code = "es-code-missing";
            if ($woocommerce_checkbox == 'yes'){
                //es_days_valid only if its eBook
                $days_valid = isset($_POST['_es_days_valid'])?intval($_POST['_es_days_valid']):intval($days_valid);
                $cloud_book_code = isset($_POST['_es_cloud_book_code'])?$_POST['_es_cloud_book_code']:$cloud_book_code;
            }
            $product->update_meta_data( '_es_days_valid', $days_valid );
            $product->update_meta_data( '_es_cloud_book_code', $cloud_book_code );
            
         }else{
             error_log("ES : Removing is_es_ebook & es_days_valid from previously none simple parent product");
             delete_post_meta($product->get_id(), '_is_es_ebook');
             delete_post_meta($product->get_id(), '_es_days_valid');
             //book code not removed. no need to type again if human mistake. 
             //TBD! need! we check first in product when putting in order item!
         }

     }
     
     
     ////////////// VARIATION /////////////////////////////
     
     //add eBook - option to each variation.
     public function admin_add_es_ebook_variation_product_option($loop, $variation_data, $variation){
        $is_es_ebook = (get_post_meta($variation->ID, '_is_es_ebook', true)) ? ' checked' : '';
        ?>
        <label class="tips" data-tip="<?php esc_attr_e( 'Enable this option if include eBook', 'woocommerce' ); ?>">
            <?php esc_html_e( 'eBook', 'woocommerce' ); ?>
            <input type="checkbox" class="checkbox variable_is_es_ebook" name="_is_es_ebook[<?php echo esc_attr( $variation->ID ); ?>]" <?php echo $is_es_ebook;?>/>
        </label>
        <?php
    }
    
    public function es_add_variation_custom_fields ($loop, $variation_data, $variation ){
        //https://codeinform.com/how-to-add-fields-to-woocommerce-variable-product/

        woocommerce_wp_text_input( 
          array( 
            'id'          => '_es_cloud_book_code[' . $variation->ID . ']', 
            'label'       => __( 'Unique ES eBook cloud code *', 'woocommerce' ), 
            'desc_tip'    => 'true',
            'type' => 'text',
            'description' => __( 'Enter the unique code you gave to this eBook on EpubSystems cloud', 'woocommerce' ),
            'value'       => get_post_meta( $variation->ID, '_es_cloud_book_code', true )
          )
        );
        
        woocommerce_wp_text_input( 
          array( 
            'id'          => '_es_days_valid[' . $variation->ID . ']', 
            'label'       => __( 'eBook Validity (days)', 'woocommerce' ), 
            'desc_tip'    => 'true',
            'type' => 'number',
            'description' => __( 'Enter number of days the eBook will exist in user\'s cloud. empty if regular sale with no limit.', 'woocommerce' ),
            'value'       => get_post_meta( $variation->ID, '_es_days_valid', true ),
            'custom_attributes' => array(
                    'step'  => '1',
                    'min' => '0'
                  ) 
          )
        );
        
        
    }
    
    //save variation
    public function es_save_variation_product_option($variation_id){
        //error_log("variation save ... ". $variation_id. " | ". $i);
        $variation_is_ebook = false;
        if ( isset( $_POST['_is_es_ebook'] ) ){
            //error_log(print_r($_POST['_is_es_ebook'], true));
            foreach ( $_POST['_is_es_ebook'] as $productid=>$checked ){
                if ($productid == $variation_id){
                    $variation_is_ebook = true;
                    //error_log("variation ".$variation_id." is_es_ebook checked : yes ");
                    update_post_meta( $variation_id, '_is_es_ebook', 'yes' );
                } 
            }

        }
        
        //save es_days_valid only if ebook
        if ($variation_is_ebook){
            //https://codeinform.com/how-to-add-fields-to-woocommerce-variable-product/
            // Number Field
            $days_valid = $_POST['_es_days_valid'][ $variation_id ];
            if( ! empty( $days_valid ) ) {
              update_post_meta( $variation_id, '_es_days_valid', esc_attr( intval($days_valid) ) );
            }else{
              update_post_meta( $variation_id, '_es_days_valid', 0 );//no limit
            }
            
            $book_code = $_POST['_es_cloud_book_code'][ $variation_id ];
            if( ! empty( $book_code ) ) {
              update_post_meta( $variation_id, '_es_cloud_book_code', esc_attr( $book_code ) );
            }else{
              update_post_meta( $variation_id, '_es_cloud_book_code', "es-code-missing" );
            }
        }//if not eBook, we remove any value from DB
        else{
            error_log("ES : Removing is_es_ebook & es_days_valid from previously variations");
            delete_post_meta( $variation_id, '_is_es_ebook');
            delete_post_meta( $variation_id, '_es_days_valid');
            //not removing book code. maybe just human mistake
            //error_log("variation ".$variation_id." is_es_ebook checked : no ");
            
        }
    }
    
    
    //////////////////////////// plugin internal actions ///////////////////////////
    
    private function run_es_sku_to_ebook_code(){
        $product_page = 1;
        $product_per_page = 100;
        $product_query = new WP_Query( array(
            'posts_per_page' => $product_per_page,
            'paged'          => $product_page,
            'post_type'      => 'product',
            'fields'         => 'id=>parent'
        ) );
        //error_log (count($posts)." products in the system...");
        $count_copied = 0;
        $copied_csv = array();
        while($product_query->have_posts() ) {
            $post = $product_query->next_post();
            
            error_log("SKU to eBook-Code: ".$count_copied." product proccessed");
            
            //proccess
            $product = wc_get_product( $post->ID );
            $copied_csv_fields = array();
            
            $sifriShopSupplierSKU = 0;
            if (function_exists("pods")){
                $product_pod = pods('product', $product->get_id());
                $sifriShopSupplierSKU = $product_pod->field('supplier_sku');
            }

            //variable products
            if ($product->is_type('variable')) {
                $args = array(
                    'post_parent' => $product->get_id(),
                    'post_type'   => 'product_variation',
                    'numberposts' => -1,
                );
                $variations = $product->get_available_variations();
                //for sifriapp shop
                foreach($variations as $var):
                    $variation = wc_get_product($var['variation_id']);
                    $sku = (strlen($sifriShopSupplierSKU)>1)?$sifriShopSupplierSKU:(strlen($variation->get_sku()) > 1?$variation->get_sku():"");
                    if (strlen($sku) > 1){
                        //error_log ($variation->get_formatted_name());
                        $copied_csv_fields['title'] = $variation->get_formatted_name();
                        $copied_csv_fields['status'] = "copied";
                        $count_copied++;
                        $variation->update_meta_data( '_es_cloud_book_code', $sku );
                        $variation->save_meta_data();
                    }else{
                        //error_log ($variation->get_formatted_name() . "NON virtual");
                        $copied_csv_fields['title'] = $variation->get_formatted_name();
                        $copied_csv_fields['status'] = "MISSING SKU";
                    }
                endforeach;
            }

            //simple products
            else if ($product->is_type('simple')) {
                $sku = (strlen($sifriShopSupplierSKU)>1)?$sifriShopSupplierSKU:(strlen($product->get_sku()) > 1?$product->get_sku():"");
                if (strlen($sku) > 1){
                    //error_log ($product->get_formatted_name());
                    $copied_csv_fields['title'] = $product->get_formatted_name();
                    $copied_csv_fields['status'] = "copied";
                    $count_copied++;
                    $product->update_meta_data( '_es_cloud_book_code', $sku );
                    $product->save_meta_data();

                }else{
                    $copied_csv_fields['title'] = $product->get_formatted_name();
                    $copied_csv_fields['status'] = "MISSING SKU";
                }
            }
            else{
                $copied_csv_fields['title'] = $product->get_formatted_name();
                $copied_csv_fields['status'] = "not simple or variation product";
            }
            
            
            $copied_csv[] = $copied_csv_fields;

            if(!$product_query->have_posts()){
                // get the next lot of results.
                $product_page++;
                $product_query = new WP_Query( array(
                    'posts_per_page' => $product_per_page,
                    'paged'          => $product_page,
                    'post_type'      => 'product'
                ) );
            }
        //endforeach;
        }
        
  
        //better this ajax call
        $data = array();
        $data["csv_link"] = $this->getCSVlink("sku_copied_to_ebook_code_report", array("title", "status"), $copied_csv);
        echo json_encode(array('message' => $count_copied." products SKUs copied to eBook", 'data' => $data));
        wp_die();
    }
    
    ///////////////////////////////////////////////////////////////////////////////////////
    
    //handle custom actions from plugin page
    private function run_es_virtual_to_ebook(){

        $product_page = 1;
        $product_per_page = 100;
        $product_query = new WP_Query( array(
            'posts_per_page' => $product_per_page,
            'paged'          => $product_page,
            'post_type'      => 'product',
            'fields'         => 'id=>parent'
        ) );
        //error_log (count($posts)." products in the system...");
        $virtualConverted = 0;
        $converted_csv = array();
        //foreach($posts as $post):
        while($product_query->have_posts() ) {
            $post = $product_query->next_post();
   
            error_log("Virtual to eBook: ".$virtualConverted." product proccessed");
            
            //proccess
            $product = wc_get_product( $post->ID );
            $converted_csv_fields = array();

            //variable products
            if ($product->is_type('variable')) {
                $args = array(
                    'post_parent' => $product->get_id(),
                    'post_type'   => 'product_variation',
                    'numberposts' => -1,
                );
                $variations = $product->get_available_variations();
                foreach($variations as $var):
                    $variation = wc_get_product($var['variation_id']);
                    if ($variation->is_virtual()){
                        //error_log ($variation->get_formatted_name());
                        $converted_csv_fields['title'] = $variation->get_formatted_name();
                        $converted_csv_fields['type'] = "variation";
                        $virtualConverted++;
                        $variation->update_meta_data( '_is_es_ebook', 'yes' );
                        $variation->save_meta_data();
                    }else{
                        //error_log ($variation->get_formatted_name() . "NON virtual");
                        $converted_csv_fields['title'] = $variation->get_formatted_name();
                        $converted_csv_fields['type'] = "variation. NON VIRTUAL";
                    }
                endforeach;
            }

            //simple products
            else if ($product->is_type('simple')) {
                if ($product->is_virtual()){
                    //error_log ($product->get_formatted_name());
                    $converted_csv_fields['title'] = $product->get_formatted_name();
                    $converted_csv_fields['type'] = "simple";
                    $virtualConverted++;
                    $product->update_meta_data( '_is_es_ebook', 'yes' );
                    $product->save_meta_data();

                }else{
                    $converted_csv_fields['title'] = $product->get_formatted_name();
                    $converted_csv_fields['type'] = "simple. NON VIRTUAL";
                }
            }
            else{
                $converted_csv_fields['title'] = $product->get_formatted_name();
                $converted_csv_fields['type'] = "other. Not Converted";
            }
            
            
            $converted_csv[] = $converted_csv_fields;

            
            if(!$product_query->have_posts()){
                // get the next lot of results.
                $product_page++;
                $product_query = new WP_Query( array(
                    'posts_per_page' => $product_per_page,
                    'paged'          => $product_page,
                    'post_type'      => 'product'
                ) );
            }
        //endforeach;
        }
        
        //bad practice
        //wp_redirect($_SERVER['HTTP_REFERER']."&convert=done");
        //better this ajax call
        $data = array();
        $data["csv_link"] = $this->getCSVlink("virtuals_converted_to_ebook_report", array("title", "type"), $converted_csv);
        echo json_encode(array('message' => $virtualConverted." Virtual products Converted to eBook", 'data' => $data));
        wp_die();

    }
    
    ///////////////////////////// helpers /////////////////////////////////////
    
    
     //https://cullenwebservices.com/create-a-csv-file-wordpress-user-information-with-php/
    private function getCSVlink($filename, $header, $values) {
 
        $path = wp_upload_dir();   // or where ever you want the file to go
        $outstream = fopen($path['path']."/".$filename."_".date("Y-m-d").".csv", "w");  // the file name you choose

        fputcsv($outstream, $header);  //creates the first line in the csv file

        //each is single line, only codes for now!
        foreach($values as $fields)
        {
           fputcsv($outstream, $fields); //one in row
        }
        //fputcsv($outstream, $values);  //output the user info line to the csv file

        fclose($outstream); 
        return '<a href="'.$path['url'].'/'.$filename.'_'.date("Y-m-d").'.csv">Download Report</a>';  //make a link to the file so the user can download.
    }


}
