<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://www.epubsystems.com
 * @since             1.0.0
 * @package           Epubsystems_woo
 *
 * @wordpress-plugin
 * Plugin Name:       eBook WooSell
 * Plugin URI:        https://www.epubsystems.com
 * Description:       Woocommerce plugin for distributing eBooks directly to eReaders via ePubSystems.com cloud and applications.
 * Version:           4.1.9
 * Author:            EpubSystems | Nir Louk
 * Author URI:        https://www.epubsystems.com/about
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       epubsystems_woo
 * Domain Path:       /languages
 */

/* 
* 4.1.9
* fixed php 8 error logs
* fixed landed books data and other order data add/update code
* 4.1.8
* fix Cannot use object of type WP_Error as array
*V4.1.6 
* wp version support 5.9.1
* V4.1.5
 * code clean. svn build. version checked to latest.
 * V4.1.3
 * improve memory issue when run upgrade tools
 * fix error if pods plugin not exist
 * V4.1.2
 * fixed memory issue when run upgrade tools
 * V4.1.1
 * missing files into SVN
 * V4.1.0
 * Major upgrade. Added eBook mark that replace the Woo-based is_virtual.
 * each product or variation has its own "ES Cloud Code" , instead of using the shop SKU
 * added support for landing books for X number of days. use "days valid" field.
 * on plugin settings page - 2 upgrade tools. first will set "eBook" to all products and variations that are set "Virtual". secong tool will copy product SKU into the new "ES Cloud Code" field.
 * recommended to activate the two tools ONLY ONCE, and then to start using dedicated new fields.
 * V4.0.0
 * -fix of gift email not hidden on checkbox
 * -first deliver to wordpress.org
 * version 3.0.0 -> 4.0.0
 * hook on webhook send and show admin status of delivery
 * ver 1.9
 * snitized email and major fix es_user_email on order
 * version 1.7
 * fix of deprecated methods
 * version 1.6
 * deprecated woocommerce_add_order_item_meta
 * replaced by woocommerce_checkout_create_order_line_item
 * version 1.5.0
 * widget settings - all text can be changed.
 * add ability to click to change email for gift
 *V.2.0.2
 * - href field for download app/moreinfo link on checkout page
 * V2.0
 * -admin_update_order_meta - for creating order by admin backend
 * - must add es_user_email custom field manually.
 *
 * * V1.3
 * added is_virtual for item in order.
 *
 *  */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}
/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'EPUBSYSTEMS_WOO_PLUGIN_NAME_VERSION', '4.1.9' );


//this is synced to the cloud
define( 'CLOUD_API_OK', 0 );
define( 'CLOUD_API_FAIL', 1 );
define( 'CLOUD_API_WARN', 2 );
define( 'CLOUD_NOTICE', 3 );

//this is plugin states. synced with css.
define( 'ES_WEBHOOK_STATUS_AWAIT', 'AWAIT' );

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-epubsystems_woo-activator.php
 */
function activate_epubsystems_woo() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-epubsystems_woo-activator.php';
	Epubsystems_woo_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-epubsystems_woo-deactivator.php
 */
function deactivate_epubsystems_woo() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-epubsystems_woo-deactivator.php';
	Epubsystems_woo_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_epubsystems_woo' );
register_deactivation_hook( __FILE__, 'deactivate_epubsystems_woo' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-epubsystems_woo.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_epubsystems_woo() {

	$plugin = new Epubsystems_woo();
	$plugin->run();

}
run_epubsystems_woo();
