<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://www.epubsystems.com
 * @since      1.0.0
 *
 * @package    Epubsystems_woo
 * @subpackage Epubsystems_woo/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Epubsystems_woo
 * @subpackage Epubsystems_woo/public
 * @author     Nir Louk <looknear@gmail.com>
 */
class Epubsystems_woo_Public {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of the plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct($plugin_name, $version) {

        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }

    /**
     * Register the stylesheets for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_styles() {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Epubsystems_woo_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Epubsystems_woo_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */
        wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/epubsystems_woo-public.css', array(), $this->version, 'all');
    }

    /**
     * Register the JavaScript for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts() {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Epubsystems_woo_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Epubsystems_woo_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */
        wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/epubsystems_woo-public.js', array('jquery'), $this->version, false);
    }

////////////////////////////////////////////////////////////////

    public function checkout_update_order_meta($order_id) {
        //add the buyer email - so we will know to who to give the eBook
        $gift_email = filter_input(INPUT_POST, 'es_gift_email', FILTER_SANITIZE_EMAIL);
        if (!empty($gift_email)) {
            //take gift email if exist
            update_post_meta($order_id, 'es_user_email', $gift_email);
            update_post_meta($order_id, 'es_is_gift', true);
        } else {
            update_post_meta($order_id, 'es_is_gift', false);
            //or take the logged user email
            $user_id = get_current_user_id();
            $user_info = get_userdata($user_id);
            if (false == $user_info){
                //not logged in, take the billing email
                $billing_email = filter_input(INPUT_POST, 'billing_email', FILTER_SANITIZE_EMAIL);
                update_post_meta($order_id, 'es_user_email', $billing_email);
            }else{
                update_post_meta($order_id, 'es_user_email', $user_info->user_email);
            }
        }

        //put in order the website publisher email
        //as ALL books in the order - belong / can be deliver by the website admin access
        $options = get_option($this->plugin_name);
        if (!empty($options) && !empty($options['es_publisher_email'])) {
            //admin set his email in the admin dashboard
            update_post_meta($order_id, 'es_publisher_email', $options['es_publisher_email']);
        } else {
            //just send the wordpress admin email
            update_post_meta($order_id, 'es_publisher_email', get_option('admin_email'));
        }

        //write_log("epub_woo : checkout_update_order_meta");
    }

/////////////////////////////////////////////////////////////////////////////

    public function adding_custom_data_in_order_items_meta($item, $cart_item_key, $values, $order) {

        //to support variations. we need to know this in the cloud.
        //for every item in the order
        $product = $item->get_product();
        //wc_add_order_item_meta($itemId, 'is_virtual', $product->is_virtual());
        $item->add_meta_data( 'is_virtual' , $product->is_virtual() );
        
        $item->add_meta_data( 'is_es_ebook' , get_post_meta($product->get_id(), '_is_es_ebook', true) );
        $item->add_meta_data( 'es_days_valid' , get_post_meta($product->get_id(), '_es_days_valid', true) );
        $item->add_meta_data( 'es_cloud_book_code' , get_post_meta($product->get_id(), '_es_cloud_book_code', true) );

        error_log("epub_woo : adding_custom_data_in_order_items_meta");

    }


////////////////////////////////////////////////////////////////////////

    //but this will remove it from the admin order and emails etc
    public function filter_format_order_meta_data($formatted_meta){
        $temp_metas = [];
        foreach($formatted_meta as $key => $meta) {
            if ( isset( $meta->key ) && ! in_array( $meta->key, [
                    'is_virtual',
                    'is_es_ebook',
                    'es_days_valid',
                    'es_cloud_book_code'
                ] ) ) {
                $temp_metas[ $key ] = $meta;
            }
        }
        return $temp_metas;
    }


////////////////////////////////////////////////////////////////////////////

    //add custom fields on order page on checkout, used in order data
    public function customise_checkout_field($checkout) {
        $options = get_option($this->plugin_name);
        
        if(!isset($options['es_title']) || empty($options['es_title'])){
            $es_title = "Digital Books";

        }else{
            $es_title = $options['es_title'];
        }

        if(!isset($options['es_message']) || empty($options['es_message'])){
            $es_message = "Digital books will be transfered to your private Cloud Library. You can read all your books with SifriApp eReading Application.";

        }else{
            $es_message = $options['es_message'];
        }

        if(!isset($options['es_gift_text']) || empty($options['es_gift_text'])){
            $es_gift_text = "Send as Gift";

        }else{
            $es_gift_text = $options['es_gift_text'];
        }

        if(!isset($options['es_email_placeholder']) || empty($options['es_email_placeholder'])){
            $es_email_placeholder = "type reciever email here";
        }else{
            $es_email_placeholder = $options['es_email_placeholder'];
        }

        if(!isset($options['es_link_text'])){
            $es_link_text = "download the app / more information";
        }else{
            $es_link_text = $options['es_link_text'];
        }
        if(!isset($options['es_link_href']) || empty($options['es_link_href'])){
            $es_link_href = "https://shop.sifriapp.co.il";
        }else{
            $es_link_href = $options['es_link_href'];
        }




        echo '<div id="es_options_checkout"><h2 id="es_options_checkout_title">' . $es_title . '</h2>';
        echo '<div id ="es_options_checkout_message" class="highlighted">'.nl2br($es_message).'</div>';

        echo '<input type="checkbox" id="es_gift_checkbox" name="es_gift_checkbox" value="1" onclick="es_isGift()"> <b>'.$es_gift_text.'</b>';

        woocommerce_form_field('es_gift_email', array(
            'type' => 'text',
            'class' => array('hidden','form-row-wide'),
            'placeholder' => $es_email_placeholder,
            'default' => ""
                ), $checkout->get_value('es_gift_email'));

        echo '<p id="es_link_text"><a href="'.$es_link_href.'" target="_blank">'.$es_link_text.'</a></p></div>';

        //warning to user before buy - in case plugin is not configured and we don't have a publisher
        if (empty($options) || empty($options['es_publisher_email'])) {
            echo '<div class="alert alert-warning">' .
            _e('<strong>Warning!</strong> Epubsystems settings missing.', $this->plugin_name)
            . '</div>';
        }
    }

    //////////// check custom fields exist //////////////
    public function customise_checkout_field_process()
    {
            // if field not set - show an error message.
            //if (!$_POST['es_user_email']) wc_add_notice(__('Please enter email for books delivery.') , 'error');
    }


}
